<?php

namespace Core\Exceptions;

use Throwable;
use Whoops\Handler\Handler;
use Whoops\Handler\JsonResponseHandler;
use Whoops\Handler\PlainTextHandler;
use Whoops\Handler\PrettyPageHandler;
use Whoops\Run as Whoops;

class ExceptionHandler
{
    /**
     * Exceptions reporter instance
     *
     * @var Whoops
     */
    private static Whoops $reporter;

    /**
     * Exceptions handler instance
     *
     * @var \Core\Exceptions\ExceptionHandler
     */
    private static ExceptionHandler $instance;

    /**
     * Get instance of exceptions handler
     *
     * @return \Core\Exceptions\ExceptionHandler
     */
    public static function instance(): ExceptionHandler
    {
        if (!isset(self::$instance)) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Get exceptions reporter instance
     *
     * @return \Whoops\Run
     */
    public static function reporter(): Whoops
    {
        if (!isset(self::$reporter)) {
            $reporter = new Whoops();
            $reporter->pushHandler(self::handler());

            self::$reporter = $reporter;
        }

        return self::$reporter;
    }

    /**
     * Get correct handler
     *
     * @return \Whoops\Handler\Handler
     */
    private static function handler(): Handler
    {
        if (env('debug')) {
            if (request()?->is('post')) {
                return new JsonResponseHandler();
            }

            return new PrettyPageHandler();
        }

        return new PlainTextHandler();
    }

    /**
     * Catch all exceptions
     *
     * @return void
     */
    public function register(): void
    {
        self::reporter()->register();
    }

    /**
     * Catch single exception
     *
     * @param \Throwable $exception
     * @return never
     */
    public function handle(Throwable $exception): never
    {
        self::reporter()->handleException($exception);

        exit(0);
    }

    /**
     * Make new exception
     *
     * @param null        $abstract
     * @param string|null $message
     * @return never
     */
    public function make(mixed $abstract = null, string|null $message = null): never
    {
        if(is_string($abstract)) {
            $abstract = app()->make($abstract, $message);
        }

        if(is_subclass_of($abstract, 'Exception')) {
            $this->handle($abstract);
        }

        exit(0);
    }

}