<?php

use Core\Bootstrap;
use Core\Env\Env;
use Core\Exceptions\ExceptionHandler;
use Core\Http\Request;


if(!function_exists('app'))
{
    /**
     * Make abstract of instance or get application instance
     *
     * @param mixed $abstract
     * @param mixed $arguments
     * @return \Core\Factory\BootstrapFactory|mixed
     */
    function app(mixed $abstract = null, mixed $arguments = []): mixed
    {
        if (is_null($abstract)) {
            return Bootstrap::getInstance();
        }

        return Bootstrap::getInstance()->make($abstract, $arguments);
    }
}

if(!function_exists('env'))
{
    /**
     * Get env variable
     *
     * @param string $key
     * @return bool
     */
    function env(string $key): mixed
    {
        return Env::get($key);
    }
}

if (!function_exists('request')) {
    /**
     * Get the request instance
     *
     * @return \Core\Http\Request
     */
    function request(): Request
    {
        return app()->request();
    }
}

if (!function_exists('exceptions')) {
    /**
     * Get error handler instance
     *
     * @return \Core\Exceptions\ExceptionHandler
     */
    function exceptions(): ExceptionHandler
    {
        return ExceptionHandler::instance();
    }
}
